# Copyright 2022 neomadas-dev
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#   3. Neither the name of the copyright holder nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
from unittest import TestCase


class TemplateLibraryTestCase(TestCase):

    SETTINGS_ENVNAME = "DJANGO_SETTINGS_MODULE"

    def setUp(self):
        self.settings = os.environ.get(self.SETTINGS_ENVNAME, "")
        os.environ[self.SETTINGS_ENVNAME] = "tests.kit_settings"

    def tearDown(self):
        os.environ[self.SETTINGS_ENVNAME] = self.settings

    def test_directtag(self):
        from django.template import Context
        from django.template.engine import Engine

        from neom.kit.template.library import Library

        library = Library()

        @library.directtag
        def dummy(foo: str, bar: str = "bar"):
            return f"{foo}-{bar}"

        engine = Engine()
        engine.template_libraries["test_library"] = library

        html = engine.from_string(
            "{% load test_library %}{% dummy 'foo' %}{% dummy 'oof' 'baz' %}"
        ).render(Context())

        self.assertEqual(html, "foo-baroof-baz")
